import requests
from bs4 import BeautifulSoup
import time
import random
import re
import os


def get_article_content(article_url):
    """
    브런치 글 URL을 받아 콘텐츠를 추출하는 함수
    """
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language': 'ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7',
        'Referer': 'https://brunch.co.kr/'
    }
    
    try:
        response = requests.get(article_url, headers=headers)
        response.raise_for_status()  # 요청이 성공했는지 확인
        
        soup = BeautifulSoup(response.text, 'html.parser')
        
        # 제목 추출
        title = soup.select_one('h1.cover_title')
        title_text = title.text.strip() if title else "제목 없음"
        
        # 작성자 추출
        author = soup.select_one('span.author_name')
        author_text = author.text.strip() if author else "작성자 미상"
        
        # 본문 내용 추출 (브런치의 경우 article 태그 내부에 있는 경우가 많음)
        content_divs = soup.select('div.item_view article p, div.item_view article h1, div.item_view article h2, div.item_view article h3')
        if not content_divs:
            content_divs = soup.select('div.wrap_body p, div.wrap_body h1, div.wrap_body h2, div.wrap_body h3')
        
        content = []
        for div in content_divs:
            text = div.get_text(strip=True)
            if text:
                content.append(text)
                
        # 만약 위 선택자로 내용을 찾지 못했다면 다른 방법 시도
        if not content:
            article_content = soup.select_one('.wrap_body')
            if article_content:
                content = [p.text.strip() for p in article_content.find_all(['p', 'h1', 'h2', 'h3']) if p.text.strip()]
        
        return {
            'title': title_text,
            'author': author_text,
            'content': '\n\n'.join(content),
            'url': article_url
        }
        
    except requests.exceptions.RequestException as e:
        print(f"요청 중 오류 발생: {e}")
        return None
    except Exception as e:
        print(f"처리 중 오류 발생: {e}")
        return None


def save_article(article_data, folder='brunch_articles'):
    """
    추출한 글을 파일로 저장하는 함수
    """
    if not article_data:
        return False
    
    # 폴더가 없으면 생성
    if not os.path.exists(folder):
        os.makedirs(folder)
    
    # 파일명 생성 (특수문자 제거)
    filename = re.sub(r'[\\/*?:"<>|]', "", article_data['title'])
    filename = f"{filename}_{article_data['author']}.txt"
    filepath = os.path.join(folder, filename)
    
    try:
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write(f"제목: {article_data['title']}\n")
            f.write(f"작성자: {article_data['author']}\n")
            f.write(f"원본 URL: {article_data['url']}\n\n")
            f.write("=" * 50 + "\n\n")
            f.write(article_data['content'])
        
        print(f"파일 저장 완료: {filepath}")
        return True
    except Exception as e:
        print(f"파일 저장 중 오류 발생: {e}")
        return False


def crawl_brunch_magazine(magazine_url):
    """
    브런치 매거진의 모든 글을 크롤링하는 함수
    """
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
    }
    
    try:
        response = requests.get(magazine_url, headers=headers)
        response.raise_for_status()
        
        soup = BeautifulSoup(response.text, 'html.parser')
        
        # 글 링크 추출 (이 부분은 브런치 사이트 구조에 따라 조정 필요)
        article_links = soup.select('a.link_post')
        article_urls = []
        
        for link in article_links:
            if 'href' in link.attrs:
                url = link['href']
                if not url.startswith('http'):
                    url = f"https://brunch.co.kr{url}"
                article_urls.append(url)
        
        print(f"총 {len(article_urls)}개의 글을 발견했습니다.")
        
        # 각 글 처리
        for i, url in enumerate(article_urls):
            print(f"[{i+1}/{len(article_urls)}] 처리 중: {url}")
            article_data = get_article_content(url)
            if article_data:
                save_article(article_data)
            
            # 서버 부하를 줄이기 위해 요청 간 간격 두기
            time.sleep(random.uniform(1.5, 3.0))
            
    except Exception as e:
        print(f"매거진 크롤링 중 오류 발생: {e}")


# 사용 예시
if __name__ == "__main__":
    # 단일 글 크롤링
    article_url = "https://brunch.co.kr/@글주소"  # 실제 브런치 글 URL로 변경
    article_data = get_article_content(article_url)
    if article_data:
        save_article(article_data)
    
    # 매거진 전체 크롤링
    # magazine_url = "https://brunch.co.kr/@매거진주소"  # 실제 브런치 매거진 URL로 변경
    # crawl_brunch_magazine(magazine_url)
